// register.c
//
// Copyright (C) 1998 Jens Winkler <jwin@gmx.net>
//
// CODEC register access routines
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//

#include <KernelExport.h>
#include <Drivers.h>

#include "audio.h"
#include "adi.h"

extern isa_module_info	*isa_bus;
extern wss_info			wss;


// get indirect register
uint32
get_ireg(uint8 index)
{
  uint32 ret;

  OUTB(wss.port, index);     // set indirect address
  ret  = INB(wss.port+2);    // get low byte
  ret |= INB(wss.port+3)<<8; // get high byte

  return ret;
}


// set indirect register
void
set_ireg(uint8 index, uint32 data)
{
  OUTB(wss.port, index);     // set indirect address
  OUTB(wss.port+2, data);    // set low byte
  OUTB(wss.port+3, data>>8); // set high byte
}


// set bits in codec indirect register
void
change_ireg_bits(uint8 index, uint32 mask, uint32 data)
{
  uint32 tmp = ~mask;

  data &= mask;                 // mask data

  acquire_sem(wss.reg_lock);    // acquire register access

  tmp &= get_ireg(index);       // read indirect register

  set_ireg(index, tmp|data);    // write indirect register

  release_sem_etc(wss.reg_lock, // release register access
                  1, B_DO_NOT_RESCHEDULE);
}


// get bits from codec register
uint32
get_ireg_bits(uint8 index, uint32 mask)
{
  uint32 ret;

  acquire_sem(wss.reg_lock);    // acquire register access

  ret = get_ireg(index);        // read indirect register

  release_sem_etc(wss.reg_lock, // release register access
                  1, B_DO_NOT_RESCHEDULE);

  return mask & ret;            // mask value and return
}
